from http import HTTPStatus
from typing import Any, Optional, Union

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_validation_error import HTTPValidationError
from ...models.prompt_sample_i_ds import PromptSampleIDs
from ...types import UNSET, Response


def _get_kwargs(project_id: str, *, run_ids: list[str]) -> dict[str, Any]:
    params: dict[str, Any] = {}

    json_run_ids = run_ids

    params["run_ids"] = json_run_ids

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": f"/projects/{project_id}/runs/prompts/rows/columnar",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Optional[Union[HTTPValidationError, PromptSampleIDs]]:
    if response.status_code == 200:
        response_200 = PromptSampleIDs.from_dict(response.json())

        return response_200
    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: Union[AuthenticatedClient, Client], response: httpx.Response
) -> Response[Union[HTTPValidationError, PromptSampleIDs]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    project_id: str, *, client: AuthenticatedClient, run_ids: list[str]
) -> Response[Union[HTTPValidationError, PromptSampleIDs]]:
    """Get Multi Run Sample Ids

     Given a list of runs with the same inputs, output the indices of each sample in each run.

    Args:
        project_id (str):
        run_ids (list[str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[HTTPValidationError, PromptSampleIDs]]
    """

    kwargs = _get_kwargs(project_id=project_id, run_ids=run_ids)

    response = client.get_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


def sync(
    project_id: str, *, client: AuthenticatedClient, run_ids: list[str]
) -> Optional[Union[HTTPValidationError, PromptSampleIDs]]:
    """Get Multi Run Sample Ids

     Given a list of runs with the same inputs, output the indices of each sample in each run.

    Args:
        project_id (str):
        run_ids (list[str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[HTTPValidationError, PromptSampleIDs]
    """

    return sync_detailed(project_id=project_id, client=client, run_ids=run_ids).parsed


async def asyncio_detailed(
    project_id: str, *, client: AuthenticatedClient, run_ids: list[str]
) -> Response[Union[HTTPValidationError, PromptSampleIDs]]:
    """Get Multi Run Sample Ids

     Given a list of runs with the same inputs, output the indices of each sample in each run.

    Args:
        project_id (str):
        run_ids (list[str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[Union[HTTPValidationError, PromptSampleIDs]]
    """

    kwargs = _get_kwargs(project_id=project_id, run_ids=run_ids)

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    project_id: str, *, client: AuthenticatedClient, run_ids: list[str]
) -> Optional[Union[HTTPValidationError, PromptSampleIDs]]:
    """Get Multi Run Sample Ids

     Given a list of runs with the same inputs, output the indices of each sample in each run.

    Args:
        project_id (str):
        run_ids (list[str]):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Union[HTTPValidationError, PromptSampleIDs]
    """

    return (await asyncio_detailed(project_id=project_id, client=client, run_ids=run_ids)).parsed
